function ImportantMap = mpaint(varargin)
% Works in a similar way to the Windows Paint program (except for the ROI image manipulations)
%
% USAGE:
%   MPAINT operates on the current figure.
%   MPAINT(HANDLE) operates in the single image contained in HANDLE.
%   HANDLE can be a handle to a figure, axes, or image object.
%   For the floodfill painting, every left-click paint a group of connected pixels.
%   (see below the 'connectivity' and 'tolerance'). A right click stops the painting.
%   For all other operations a left-click starts the painting, which continues for
%   every mouth movement and stops whith any mouse click.
%
%   H = MPAINT(...); returns the handles of the MPAINT figure
%
%   MPAINT works with both RGB and indexed images but the later are converted to RGB using
%   the highly ineficient and memory consuming 'ind2rgb' Matlab base function.
%   
%   Specifying connectivity:
%   By default, MPAINT uses 4-connected neighbors but you can select
%   8-connected in the apropriate radio buttons
%
%   Specifying tolerance:
%   Tolerance is used only when the doing painting operations (when using the bucket icon)
%   The "Tolerance" slider selects the maximal lower/higher brightness/color difference
%   between the currently hited pixel and its neighbors. Pixels within the -> hit pixel
%   +- |tolerance| are considered to belong to the repainted domain.
%
%   NOTE1: MPAINT is an GUI interface to some functions in the CVLIB_MEX
%   library, so you must have it installed to run this function
%
% EXAMPLES:
%       load clown
%       h = imagesc(X); axis image
%       set(gcf,'colormap',map,'DoubleBuffer','on')
%       mpaint(h);                      % Click at will
%       B = getimage(h);                % Get your work of art

%   AUTHOR
%       Joaquim Luis  - 19-March-2007
%       jluis@ualg.pt - Universidade do Algarve
%

% M-File changed by desGUIDE 
global Gbrushwidth;
global Genergy;
global ImportantMap;
global cancel_flag;
ImportantMap = varargin{3};

hObject = figure('Tag','figure1','Visible','off');
handles = guihandles(hObject);
guidata(hObject, handles);
mpaint_LayoutFcn(hObject,handles);
handles = guihandles(hObject);
 
% Import icons
load('IcoChris.mat')

h_toolbar = uitoolbar('parent',hObject, 'BusyAction','queue','HandleVisibility','on',...
   'Interruptible','on','Tag','FigureToolBar','Visible','on');
uipushtool('parent',h_toolbar,'Click',{@restore_clickedcallback},'cdata',Refresh_ico,'Tooltip','Restore Image','Sep','on');
uipushtool('parent',h_toolbar,'Click',{@load_clickedcallback},'cdata',Load_ico,'Tooltip','Load Map','Sep','on');
uipushtool('parent',h_toolbar,'Click',{@save_clickedcallback},'cdata',Save_ico,'Tooltip','Save Map','Sep','on');

if (~isempty(varargin))
    handles.hCallingFig = varargin{1};
else
    handles.hCallingFig = gcf;
end

%%%%%%%%%%%%%%%%%
if (~isempty(varargin))
    handles.hCallingFig2 = varargin{2};
else
    handles.hCallingFig2 = gcf;
end
%%%%%%%%%%%%%%%%%%%%%
if (~ishandle(handles.hCallingFig))
    errordlg('Error, first argument is not a valid handle','ERROR');
    delete(hObject);    return
end

if (~ishandle(handles.hCallingFig2))
    errordlg('Error, second argument is not a valid handle','ERROR');
    delete(hObject);    return
end

if ( strcmp(get(handles.hCallingFig,'type'),'axes') || strcmp(get(handles.hCallingFig,'type'),'figure'))
    handles.hImage = findobj(handles.hCallingFig,'type','image');
elseif ( strcmp(get(handles.hCallingFig,'type'),'image') )
    handles.hImage = handles.hCallingFig;
end

if ( strcmp(get(handles.hCallingFig2,'type'),'axes') || strcmp(get(handles.hCallingFig2,'type'),'figure'))
    handles.SalienceMap = findobj(handles.hCallingFig2,'type','image');
elseif ( strcmp(get(handles.hCallingFig,'type'),'image') )
    handles.SalienceMap = handles.hCallingFig2;
end

if (isempty(handles.hImage))
    errordlg('Error, no image found','ERROR');
    delete(hObject);    return
end

if (isempty(handles.SalienceMap))
    errordlg('Error, no Salience Map found','ERROR');
    delete(hObject);    return
end
% Ok now we will fish the tue fig handle (before it could have been an img handle for example)
handles.hCallingFig = get(get(handles.hImage,'Parent'),'Parent');
handles.hCallingFig2 = get(get(handles.SalienceMap,'Parent'),'Parent');
set(handles.hCallingFig, 'NumberTitle', 'off', 'Name','Paint Window: Click Mouse to start');
set(handles.hCallingFig2, 'NumberTitle', 'off', 'Name','Use the Painter pannel to refresh or save');

% Try to position this figure glued to the right of calling figure
posThis = get(hObject,'Pos');
posParent = get(handles.hCallingFig,'Pos');
ecran = get(0,'ScreenSize');
xLL = 50;
yLL = ecran(4) - 150;
set(hObject,'Pos',[xLL yLL posThis(3:4)])

handles.hCallingAxes = get(handles.hCallingFig,'CurrentAxes');
handles.XData = get(handles.hImage,'XData');
handles.YData = get(handles.hImage,'YData');
handles.origImg = get(handles.hImage,'CData');        % Make a copy of the original image
handles.origSalienceMap = get(handles.SalienceMap,'CData');        % Make a copy of the original image
handles.imgSize = size(handles.origImg);


handles.connect = 4;
handles.tol = 20;
handles.fillColor = uint8([255 255 255]);
handles.lineWidth = 11;
handles.elemSquare = 0;
handles.buttonFlag = 1;
handles.startNewBrush = 1;
Gbrushwidth = handles.lineWidth;
Genergy = 1;

set(handles.slider_LineWidth,'Value',Gbrushwidth);
set(handles.slider_BrushEnergy,'Value',Genergy*10);
% Choose default command line output for mpaint_export
% if (nargout),   varargout{1} = hObject;     end

guidata(hObject, handles);
set(hObject,'Visible','on');

% set window response function to the paint window
opt = 'paintbrush';
set(handles.hCallingFig,'WindowButtonDownFcn',{@ChrisBrush, handles, opt});
cancel_flag = 0;
set(handles.hCallingFig,'KeyPressFcn',{@ChrisCancel, handles, opt});


% --------------------------------------------------------------------
% a function to response key press in paint window (cancel paint) 
function ChrisCancel(obj,eventdata, handles, opt)
global cancel_flag;
cancel_flag = 1;


% a function to response mouse clik in paint window (start to paint) 
function ChrisBrush(obj,eventdata, handles, opt)
    set(handles.hCallingFig, 'NumberTitle', 'off', 'Name','Paint Window: Mouse Up to stop');
    figure(handles.hCallingFig2);
    figure(handles.hCallingFig);
    state = uisuspend(handles.hCallingFig);        % Remember initial figure state
    if (strcmp(opt,'pencil'))
        set(handles.hCallingFig,'Pointer', 'custom','PointerShapeCData',getPointer('pencil'),'PointerShapeHotSpot',[14 2])
    elseif (strcmp(opt,'paintbrush'))
        set(handles.hCallingFig,'Pointer', 'custom','PointerShapeCData',getPointer('brush'),'PointerShapeHotSpot',[14 2])
    end

    global Gbrushwidth;
    lineThick = Gbrushwidth;
    lineType = 16;
    % get the size and sd for gaussian filter for the brush
%     f_size = 3*lineThick;
%     if f_size > 33
%         f_size = 33;
%     end
%     f_sd = (lineThick/5)^3;
%     if f_sd > 33
%         f_sd = 33;
%     end
    f_size = 2*lineThick;
    if f_size > 23
        f_size = 23;
    end
    f_sd = (lineThick/5)^3;
    if f_sd > 33
        f_sd = 33;
    end
    fm_size = 3*ceil(lineThick/11);
    fm_sd = 1;
    h = fspecial('gaussian', f_size, f_sd);
    hm = fspecial('gaussian', fm_size, fm_sd);
    paintFirstButtonDown(handles,state,lineType, lineThick, h, hm);

% --------------------------------------------------------------------
function restore_clickedcallback(hObject, eventdata)
    % Restore original (well its RGB version if it was indexed) image
    handles = guidata(hObject);     % get handles
    set(handles.hImage,'CData',handles.origImg);
    set(handles.SalienceMap,'CData',handles.origSalienceMap);
    
    % --------------------------------------------------------------------
function save_clickedcallback(hObject, eventdata)
    % Restore original (well its RGB version if it was indexed) image
    [fname,pname] = uiputfile_name('*.bmp','Save Map');
    handles = guidata(hObject);     % get handles
    map = get(handles.SalienceMap,'CData');
    [pname fname]
    imwrite(map, [pname fname], 'bmp');
    close all; 

function load_clickedcallback(hObject, eventdata)
    % Restore original (well its RGB version if it was indexed) image
    [fname,pname] = uigetfile_name('*.bmp','Load Map');
    global ImportantMap;
    ImportantMap = imread([pname fname]);
    close all;
    
% -------------------
function paintFirstButtonDown(handles,state,lineType, lineThick, h, hm)
    % declare window functions for mouse move and mouse up in paint window
    pt = get(handles.hCallingAxes, 'CurrentPoint');
    setappdata(handles.figure1,'prev_pt',pt(1,1:2))
    set(handles.hCallingFig,'WindowButtonMotionFcn',{@wbm_line,handles,lineThick,lineType, h, hm},...
                'WindowButtonUpFcn',{@wbd_paint,handles.hCallingFig, handles, state});
    if (~handles.elemSquare && lineThick > 1)       % Use a round element
        set(handles.hCallingFig,'WindowButtonMotionFcn',{@wbm_circ, handles, lineThick, lineType, h, hm});
        set(handles.hCallingFig, 'WindowButtonUpFcn', {@wbd_paint, handles.hCallingFig, handles, state, hm});
    end
% -------------------
function wbm_line(obj,eventdata,handles,lineThick,lineType, h, hm)
    % Draw the line using a square element
    pt = get(handles.hCallingAxes, 'CurrentPoint');
    prev_pt = getappdata(handles.figure1,'prev_pt');
    setappdata(handles.figure1,'prev_pt',pt(1,1:2))
    x = round( localAxes2pix(handles.imgSize(2),handles.XData,[prev_pt(1) pt(1,1)]) );
    y = round( localAxes2pix(handles.imgSize(1),handles.YData,[prev_pt(2) pt(1,2)]) );
    if (~insideRect(handles,x(2),y(2))),      return;     end
    win_dx = abs(diff(x)) + 4;         win_dy = abs(diff(y)) + 4;
    
    % Notice that the (2) index denotes the current point
    win_left = max(x(2)-win_dx,1);      win_right = min(x(2)+win_dx,handles.imgSize(2));
    win_top  = max(y(2)-win_dy,1);      win_bot   = min(y(2)+win_dy,handles.imgSize(1));
    if (win_top > win_bot)
        fds = win_top;        win_top = win_bot;        win_bot = fds;
    end
       
    r = win_top:win_bot;    c = win_left:win_right;
    img = get(handles.hImage,'CData');
    map = get(handles.SalienceMap,'CData');
    img_s = img(r,c,:);     % Extract a sub-image
    x = x - c(1);           % We need the PTs coords relative to the subimage
    y = y - r(1);

    cvlib_mex('line',uint8(img_s),[x(1) y(1)],[x(2) y(2)],handles.fillColor,lineThick,lineType);
    img(r, c, :) = img_s;
    map(r, c, 1) = img_s(:, :, 1);
    set(handles.hImage,'CData',img);
    set(handles.SalienceMap,'CData',map);

    
% -------------------
function wbm_circ(obj,eventdata,handles,lineThick,lineType, h, hm)
    global Gbrushwidth;
    global Genergy;
    % Draw the line using a circular element
    pt = get(handles.hCallingAxes, 'CurrentPoint');
    x = round( localAxes2pix(handles.imgSize(2),handles.XData,pt(1,1)) );
    y = round( localAxes2pix(handles.imgSize(1),handles.YData,pt(1,2)) );
    if (~insideRect(handles,x,y)),      return;     end
    
    win_left = max(x-lineThick-1,1);      win_right = min(x+lineThick+1,handles.imgSize(2));
    win_top  = max(y-lineThick-1,1);      win_bot   = min(y+lineThick+1,handles.imgSize(1));
    if (win_top > win_bot)
        fds = win_top;        win_top = win_bot;        win_bot = fds;
    end

    r = win_top:win_bot;    c = win_left:win_right;
    img = get(handles.hImage,'CData');
    map = get(handles.SalienceMap,'CData');
    img_s = img(r,c,:);     % Extract a sub-image
    x = x - c(1);           y = y - r(1);      % We need the PTs coords relative to the subimage
    
    % brush tool from openCV
    cvlib_mex('circle',uint8(img_s),[x y],lineThick,handles.fillColor,-1,lineType);

    % find the region that has been changed
    clipmask = img(r,c,1) == img_s(:, :, 1) & img(r,c,2) == img_s(:, :, 2) & img(r,c,3) == img_s(:, :, 3);
    clipmask = repmat(clipmask, [1, 1, 3]);
    img_s = 255*imfilter(double(clipmask), h, 'replicate');
    weight = 0.01*Gbrushwidth;
    if weight < 0.1
        weight = 0.1;
    else if weight > 0.7
        weight = 0.7;
        end
    end
    energy = Genergy;
    imgg = double(img(r,c,:)).*((double(img_s(:, :, :))/255).^weight) + 255*energy*ones(size(img(r,c,:))).*(1 - ((double(img_s(:, :, :))/255).^weight));
    img(r,c,:) = double(img(r, c, :)).*double(clipmask) + double(imgg).*double(1 - clipmask);
    map2(r,c) = double(map(r,c)).*((double(img_s(:, :, 1))/255).^weight) + 255*energy*ones(size(map(r,c))).*(1 - ((double(img_s(:, :, 1))/255).^weight));
    map(r,c) = double(map(r,c)).*double(clipmask(:, :, 1)) + double(map2(r, c)).*double(1 - clipmask(:, :, 1));
    map(r,c) = imfilter(double(map(r,c)), hm, 'replicate');
    set(handles.hImage,'CData',img);
    set(handles.SalienceMap,'CData',map);
    global ImportantMap;
    ImportantMap = map;
% -------------------
function wbd_paint(obj,eventdata,hCallFig, handles, state, hm)
    set(handles.hCallingFig, 'NumberTitle', 'off', 'Name','Paint Window: Click Mouse to start');
    uirestore(state);           % Restore the figure's initial state

% -------------------------------------------------------------------------------------
function [params,but] = prepareParams(handles, opt, opt2)
% Prepare the params structure that is to be transmited to cvlib_mex (gets also the point)
% OPT, if provided and contains a 16x16 array, is taken as a pointer
% OPT2, if provided, must be a 2 element vector with 'PointerShapeHotSpot'
% BUT returns which button has been pressed.
    if (nargin == 1),   opt = [];   opt2 = [];   end
    if (nargin == 2),   opt2 = [];  end
    params.FillColor = handles.fillColor;
    figure(handles.hCallingFig)         % Bring the figure containing image forward
    but = 1;                            % Initialize to a left click
    poin = 'crosshair';
    if (numel(opt) == 256),        poin = opt;    end
    [x,y,but]  = ginput_pointer(1,poin,opt2);
    [x,y] = getpixcoords(handles,x,y);
    params.Point = [x y];
    params.Tolerance = handles.tol;
    params.Connect = handles.connect;

% -------------------------------------------------------------------------------------
function [x,y] = getpixcoords(handles,x,y)
    % Convert x,y to pixel coordinates (they are not when the image has other coordinates)
    [m n k] = size(get(handles.hImage,'CData'));
    x = localAxes2pix(n,handles.XData,x);
    y = localAxes2pix(m,handles.YData,y);

% --------------------------------------------------------------------
function res = insideRect(handles,x,y)
    % Check if the point x,y in PIXELS is inside the rectangle RECT
    % RECT = [1 handles.imgSize(2) 1 handles.imgSize(1)]
    res = ( x >= 1 && x <= handles.imgSize(2) && y >= 1 && y <= handles.imgSize(1) );

% -------------------------------------------------------------------------------------
function toggle00_Callback(hObject, eventdata, handles)
    % All color toggle end up here
    toggleColors(hObject,handles)

% -------------------------------------------------------------------------------------
function toggleColors(hCurr,handles)
% hCurr is the handle of the current slected toggle color. Get its color
% and assign it to toggle_currColor.
    set(hCurr,'Value',1);               % Reset it to pressed state
    set(handles.toggle_currColor,'BackgroundColor',get(hCurr,'BackgroundColor'))
    handles.fillColor = round(get(hCurr,'BackgroundColor')*255);
    img = get(handles.hImage,'CData');
    if (ndims(img) == 2)            % Here we have to permanently change the image type to RGB
        %img = ind2rgb8(img,get(handles.hCallingFig,'ColorMap'));           % USE THIS IF YOU HAVE IPTX
        img = uint8(ind2rgb(img,get(handles.hCallingFig,'ColorMap'))*255);
        handles.origImg = img;      % Update the copy of the original image
        set(handles.hImage,'CData', img); 
    end
    guidata(handles.figure1,handles)

% -------------------------------------------------------------------------------------
function pushbutton_moreColors_Callback(hObject, eventdata, handles)
    c = uisetcolor;
    if (length(c) > 1)          % That is, if a color was selected
        handles.fillColor = round(c*255);
        set(handles.toggle_currColor,'BackgroundColor',c)
        img = get(handles.hImage,'CData');
        if (ndims(img) == 2)            % Here we have to permanently change the image type to RGB
            %img = ind2rgb8(img,get(handles.hCallingFig,'ColorMap'));           % USE THIS IF YOU HAVE IPTX
            img = uint8(ind2rgb(img,get(handles.hCallingFig,'ColorMap'))*255);
            handles.origImg = img;      % Update the copy of the original image
            set(handles.hImage,'CData', img); 
        end        
        guidata(handles.figure1,handles)
    end

% ---------------------------------------------------------------------------
function slider_tolerance_Callback(hObject, eventdata, handles)
    handles.tol = round(get(hObject,'Value'));
    set(handles.text_tol,'String',['Tolerance = ' num2str(handles.tol)])
    guidata(handles.figure1,handles)

% -------------------------------------------------------------------------------------
function pixelx = localAxes2pix(dim, x, axesx)
%   Convert axes coordinates to pixel coordinates.
%   PIXELX = AXES2PIX(DIM, X, AXESX) converts axes coordinates
%   (as returned by get(gca, 'CurrentPoint'), for example) into
%   pixel coordinates.  X should be the vector returned by
%   X = get(image_handle, 'XData') (or 'YData').  DIM is the
%   number of image columns for the x coordinate, or the number
%   of image rows for the y coordinate.

	xfirst = x(1);	        xlast = x(max(size(x)));
	if (dim == 1)
        pixelx = axesx - xfirst + 1;        return
	end
	xslope = (dim - 1) / (xlast - xfirst);
	if ((xslope == 1) & (xfirst == 1))
        pixelx = axesx;
	else
        pixelx = xslope * (axesx - xfirst) + 1;
	end

% -------------------------------------------------------------------------------------
function listbox_lineWidth_Callback(hObject, eventdata, handles)
    handles.lineWidth = get(hObject,'Value');
    guidata(handles.figure1, handles);

% -------------------------------------------------------------------------------------
function radio_square_Callback(hObject, eventdata, handles)
    if (~get(hObject,'Value')),      set(hObject,'Value',1);   return;     end
    handles.elemSquare = 1;
    set(handles.radio_round,'Value',0)
    guidata(handles.figure1, handles);

% -------------------------------------------------------------------------------------
function radio_round_Callback(hObject, eventdata, handles)
    if (get(hObject,'Value'))
        handles.elemSquare = 0;
        set(handles.radio_square,'Value',0)
        guidata(handles.figure1, handles);
    else
        set(hObject,'Value',1)
    end
  
% -------------------------------------------------------------------------------------------------------
function p = getPointer(opt)

if (strcmp(opt,'pipeta'))
	p = [...
		NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	1	1	1	NaN
		NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	1	NaN	NaN	NaN	1
		NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	1	NaN	1	1	1
		NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	1	1	NaN	1	1	1	1	1
		NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	1	1	1	1	1	1	1
		NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	1	NaN	1	1	1	1	NaN	NaN
		NaN	NaN	NaN	NaN	NaN	NaN	NaN	1	NaN	NaN	NaN	1	1	NaN	NaN	NaN
		NaN	NaN	NaN	NaN	NaN	NaN	1	NaN	NaN	NaN	1	1	1	NaN	NaN	NaN
		NaN	NaN	NaN	NaN	NaN	1	NaN	NaN	NaN	1	1	NaN	NaN	NaN	NaN	NaN
		NaN	NaN	NaN	NaN	1	NaN	NaN	1	1	1	NaN	NaN	NaN	NaN	NaN	NaN
		NaN	NaN	NaN	1	NaN	1	1	1	1	NaN	NaN	NaN	NaN	NaN	NaN	NaN
		NaN	NaN	1	NaN	1	1	1	1	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN
		NaN	1	NaN	1	1	1	1	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN
		NaN	1	1	1	1	1	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN
		1	1	1	1	1	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN
		NaN	1	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN];
        
elseif (strcmp(opt,'pencil'))
	p = [...
		NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN
		NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	1	1	NaN	NaN	NaN	NaN
		NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	1	NaN	NaN	1	NaN	NaN	NaN
		NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	1	NaN	NaN	NaN	NaN	1	NaN	NaN
		NaN	NaN	NaN	NaN	NaN	NaN	NaN	1	NaN	1	NaN	NaN	NaN	1	NaN	NaN
		NaN	NaN	NaN	NaN	NaN	NaN	1	NaN	NaN	NaN	1	NaN	1	NaN	NaN	NaN
		NaN	NaN	NaN	NaN	NaN	1	NaN	NaN	NaN	NaN	NaN	1	NaN	NaN	NaN	NaN
		NaN	NaN	NaN	NaN	1	NaN	NaN	NaN	NaN	NaN	1	NaN	NaN	NaN	NaN	NaN
		NaN	NaN	NaN	1	NaN	NaN	NaN	NaN	NaN	1	NaN	NaN	NaN	NaN	NaN	NaN
		NaN	NaN	1	NaN	NaN	NaN	NaN	NaN	1	NaN	NaN	NaN	NaN	NaN	NaN	NaN
		NaN	1	1	NaN	NaN	NaN	NaN	1	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN
		NaN	1	1	1	NaN	NaN	1	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN
		NaN	1	1	1	1	1	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN
		NaN	1	1	1	1	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN
		NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN
		NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN];
        
elseif (strcmp(opt,'brush'))
	p = [...
		NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	1	1	NaN
		NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	1	1	1	1
		NaN	NaN	NaN	NaN	NaN	NaN	NaN	1	1	1	NaN	1	1	1	1	1
		NaN	NaN	NaN	NaN	NaN	NaN	1	NaN	NaN	NaN	1	1	1	1	1	NaN
		NaN	NaN	NaN	NaN	NaN	1	NaN	NaN	NaN	NaN	NaN	NaN	1	1	NaN	NaN
		NaN	NaN	NaN	1	1	NaN	NaN	NaN	NaN	NaN	NaN	NaN	1	NaN	NaN	NaN
		1	1	1	NaN	NaN	NaN	NaN	NaN	NaN	1	NaN	NaN	NaN	1	NaN	NaN
		NaN	1	NaN	NaN	NaN	NaN	1	NaN	NaN	NaN	NaN	NaN	NaN	1	NaN	NaN
		NaN	NaN	1	NaN	1	NaN	NaN	NaN	NaN	NaN	NaN	NaN	1	NaN	NaN	NaN
		NaN	NaN	NaN	1	NaN	NaN	NaN	NaN	NaN	NaN	NaN	1	NaN	NaN	NaN	NaN
		NaN	NaN	NaN	NaN	1	NaN	NaN	NaN	NaN	NaN	1	NaN	NaN	NaN	NaN	NaN
		NaN	NaN	NaN	NaN	NaN	1	NaN	NaN	NaN	1	NaN	NaN	NaN	NaN	NaN	NaN
		NaN	NaN	NaN	NaN	NaN	NaN	1	1	1	NaN	NaN	NaN	NaN	NaN	NaN	NaN
		NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN
		NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN
		NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN];
elseif (strcmp(opt,'bucket'))
	p = [...
		NaN	NaN	NaN	NaN	NaN	NaN	1	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN
		NaN	NaN	NaN	NaN	NaN	1	NaN	1	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN
		NaN	NaN	NaN	NaN	NaN	1	1	1	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN
		NaN	NaN	NaN	NaN	NaN	1	NaN	1	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN
		NaN	NaN	NaN	NaN	1	NaN	NaN	1	1	NaN	NaN	NaN	NaN	NaN	NaN	NaN
		NaN	NaN	NaN	1	NaN	NaN	NaN	1	NaN	1	NaN	NaN	NaN	NaN	NaN	NaN
		NaN	NaN	1	NaN	NaN	NaN	1	NaN	1	NaN	1	NaN	NaN	NaN	NaN	NaN
		NaN	1	NaN	NaN	NaN	NaN	NaN	1	1	NaN	NaN	1	NaN	NaN	NaN	NaN
		NaN	1	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	1	1	NaN	NaN
		NaN	NaN	1	NaN	NaN	NaN	NaN	NaN	NaN	NaN	1	1	1	1	1	NaN
		NaN	NaN	NaN	1	NaN	NaN	NaN	NaN	NaN	1	NaN	NaN	NaN	1	1	NaN
		NaN	NaN	NaN	NaN	1	NaN	NaN	NaN	1	NaN	NaN	NaN	NaN	1	1	NaN
		NaN	NaN	NaN	NaN	NaN	1	1	1	NaN	NaN	NaN	NaN	NaN	1	1	NaN
		NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	1	1	NaN
		NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	1	1	NaN
		NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	NaN	1	1	NaN];
    
end

% --- Creates and returns a handle to the GUI figure. 
function mpaint_LayoutFcn(h1,handles);

set(h1,...
'Color',get(0,'factoryUicontrolBackgroundColor'),...
'MenuBar','none',...
'Name','Painter',...
'NumberTitle','off',...
'Position',[520 682 229 78],...
'RendererMode','manual',...
'Resize','off',...
'HandleVisibility','callback',...
'Tag','figure1');

uicontrol('Parent',h1,...
'BackgroundColor',[1 1 1],...
'Callback',{@mpaint_uicallback,h1,'Line_Width_Callback'},...
'Max',33,...
'Min',2,...
'Position',[10 4 210 15],...
'Style','slider',...
'SliderStep',[0.00390625 0.1],...
'TooltipString','Brush width',...
'Tag','slider_LineWidth');

uicontrol('Parent',h1,...
'HorizontalAlignment','left',...
'Position',[82 19 90 14],...
'String','Brush Width = 11',...
'Style','text',...
'Tag','text_tol');

uicontrol('Parent',h1,...
'BackgroundColor',[1 1 1],...
'Callback',{@mpaint_uicallback,h1,'Brush_Energy_Callback'},...
'Max',20,...
'Min',0,...
'Position',[10 44 210 15],...
'Style','slider',...
'SliderStep',[0.00390625 0.1],...
'TooltipString', 'BrushEnergy',...
'Tag','slider_BrushEnergy');


uicontrol('Parent',h1,...
'HorizontalAlignment','left',...
'Position',[82 60 90 14],...
'String','Brush Energy = 1',...
'Style','text',...
'Tag','text_slider_BrushEnergy');

function mpaint_uicallback(hObject, eventdata, h1, callback_name)
% This function is executed by the callback and than the handles is allways updated.
feval(callback_name,hObject,[],guidata(h1));


% -------------------------------------------------------------------------------------
function Line_Width_Callback(hObject, eventdata, handles)
handles.lineWidth = round(get(hObject,'Value'));
global Gbrushwidth;
Gbrushwidth = handles.lineWidth;
set(handles.text_tol,'String',['Brush Width = ' num2str(handles.lineWidth)])
guidata(handles.figure1, handles);
    
% -------------------------------------------------------------------------------------
function Brush_Energy_Callback(hObject, eventdata, handles)
global Genergy;
Genergy = round(get(hObject,'Value'));
set(handles.text_slider_BrushEnergy,'String',['BrushEnergy = ' num2str(Genergy/10)]);
Genergy = Genergy/10;
guidata(handles.figure1, handles);    
    