function d = jnd_distance( p1, p2, jnddata, tol )

% d = jnd_distance( p1, p2, jnddata )
% Return the jnd distance along the straight line from p1 to p2.
% Intuitively, this is defined as the maximum number of points
% we can spread along the line keeping the points about 1 jnd apart.
% But we define it as a line integral
%   int_p1^p2 dx / jnd(p)
% in which dx is step distance along the line and jnd(p) is the jnd
% at the point p. This is keeping with intuition.
% Because we have to estimate the integral the user is allowed to
% provide a tolerance.
%
% This measure is symmetric (unlike CIELAB distances). But notice
% that a sequence such as
%   f1 = jnd_estimate( jnddata, p1 ); % p1 a valid colour
%   p2 = p1 + f1*dp; % dp a unit vector, p2 a valid colour, f is 1 jnd distance
%   d = jnd_distance( p1, p2, jnddata, tol );
% will not in general give d = 1. This is because the way in which we have defined
% jnd_distance takes in account the fact that jnd varies as a function of space.
% Therefore, if we continue the above sequence
%   f2 = jnd_estimate( jnddata, p2 ); f1 ~= f2, in general
%   p3 = p1 - f2*dp; % p1 ~= p3, in general
%
% pre:
%   p1 - RGB colour, as a column vector
%   p2 - RGB colour, as a column vector
%   jnddata - the jnd model
%   tol - the tolerance of the estimate (optional; set to 0.1 be default)
% post
%   d(1) - the jnd distance between p1 and p2.

% ensure columen vector form
p1 = p1(:);
p2 = p2(:);

if nargin < 4
  tol = 0.1;
end

Npoint = 20;
dold = Inf;
d = 0;

while abs(dold-d) > tol
  Npoint = 2*Npoint;
  dold = d;
  dp = (p2 - p1)/(Npoint-1);
  %p = zeros(3,Npoint);
  f = zeros(2,Npoint);
  x = 0:(Npoint-1);
  for i = 1:Npoint
    p = p1 + dp*(i-1); % the point
    f(:,i) = jnd_estimate( p, jnddata );
  end
  d = norm(dp) * sum( 1./f(1,:) );  
  break; % just do one run - not as accurate, but ok
end


% estimate jnd given the errors too
% d(2,1) = norm(dp) *sum( 1./(f(1,:)+f(2,:)) );
% d(3,1) = norm(dp) *sum( 1./(f(1,:)-f(2,:)) );

return;




