% jnd_train.m
% ===========
%
% 1) The jnd space, f(x,y,z)
% --------------------------
% We want to model jnd distances as a function of three variables, which
% we will call x,y, and z; corresponding to three colour values.
%
% We will assume that at any point the jnd distance to the next point is
% spherically symmetric; this is unrealistic and may require refinement
% at some point in the future. This simplifying assumption allows us to
% model a scalar function, f(x,y,z), that specifies 1 jnd in Euclidean space
% at the points (x,y,z).
%
% We further assume that the function f(x,y,z) can be adequately modelled
% using (tri-)inear interpolation between given samples. An octree is used to
% as data structure to hold the samples. This allows the density of data samples
% to vary locally, there being more where the rate of change of f() is greatest;
% the response of the eye to brightness is reported as being logarithmic.
%
% The function f() is neccessarily subjective in nature; we must therefore
% estimate it empirically. Consequently we modifiy f() from a scalar function
% to a two-valued function: the empirical mean and standard deviation at
% data points choosen by a user. Both quantities are assumed to vary linearly
% between samples.
%
% 2) Empirically determining f(x,y,z)
% -----------------------------------
% The jnd function varies not only as a function of space, (x,y,z), but also
% depends upon such elements as the size (and - maybe - shape) of two touching
% regions of constant colour. We wish to use the jnd space to model noise.
% Specifically we wish to classify local colour variations as noise, if such
% variation is less than one jnd. Our experiments are, therefore, geared toward
% this end.
%
% We measure the jnd at a point (x,y,z), where no measure exists, as follows.
% First we generate a set of N^2 samples that are randomly but uniformly distributed over the surface
% of the unit sphere centred at (x,y,z). Next we estimate the jnd at (x,y,z) using
% linear intepolation, r = f(x,y,z), and scale each sample so it rests on a sphere
% of radius r, centre (x,y,z). (If we have too few samples to perform linear interpolation,
% we set r = 1). Next we plot each sample in a square test picture of
% edge 2N+1, the origin of which is top-left. The samples are plotted at locations
% (2i+1,2j+1) in the test picture; that is, locations where both coordinates are
% odd numbers. All remaining pixels are plotted with the central colour (x,y,z).
% This process ensures that every sample is seen abuting to the central colour.
% The test picture is displayed, and a user is allowed to vary the value of r,
% until a threshold value is found where the test picture _just_ becomes visually
% flat. The threshold value so found is accepted as the measured value of f() at
% (x,y,z). That is (f,x,y,z) <- r.
%
% There is a problem with this procedure: the samples to be shown should all be
% contained within the colour volume (a RGB cube of unit edge). To resolve this
% we simply continue taking random samples until all of them lie on the surface
% of a sphere, and within the volume the cube. One consequence of this is that it
% is impossible, in general to present exactly the same set of points twice in
% so that when the user changes the radius value, the picture changes too.
%
% The same procedures of measuring a threshold value can be carried out at an
% existing sample. In this case the existing threshold is updated as a running mean,
% similarly the standard deviation at the point is also updated. These updates
% require the number of times a measure has been made at the point to be recorded.
% 
% Problem: The jnd can vary between monitors, and even on the same monitor as
% colour controls (such as brighness, contrast, and gamma, etc) are ajusted.
% We therefore assume a standard setting between all monitors.

close all;
clear all;

% Control variables -------------------------------------------------------
jndfile = 'jnddata.mat';
jndfilebackup = 'jnddata.backup.mat';

tol = 1E-4;
Nedge = 49;

% Initialise progam ----------------------------------------------

if ~FileExists( jndfile )
  
  % initialise jnd data as a data structure
  jnddata = jnd_init;
  
  % initial measures in the structure
  Np = size(jnddata.func,2);
  for i = 1:8
    p = jnddata.point(:,i );
    f = jnd_measure_trial( p, tol, Nedge );
    jnddata = jnd_sample_update( f,i,jnddata, );
  end
  
  % save the jnd data
  save( jndfile, 'jnddata' );
  
  clear index i p f; % clear unwanted variables
else
  
  load( jndfile );

  % make a backup
  save( jndfilebackup, 'jnddata' );

  % add in a leaf
  jnddata = jnd_train_leaf( jnddata, tol, Nedge );
  save( jndfile, 'jnddata' );

  % iterate through the points to measure any estimates
  % these have been included by the insertion of any new leaves.
  Np = size(jnddata.func,2);
  for i = 1:Np
    if jnddata.func(3,i) == 0 % no measure here
      p = jnddata.point(:,i);
      f = jnd_measure_trial( p, tol, Nedge );
      jnddata = jnd_sample_update( f,i,jnddata );
      
      % save here to allow for use interuption etc
      save( jndfile, 'jnddata' );  
    end
  end
  

end

return;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This code is left in place because I found it useful occasionally

  % iterate through the points to measure any estimates
  % these have been included by the insertion of any new leaves.
  Np = size(jnddata.func,2);
  for i = 1:Np
    if jnddata.func(3,i) == 0 % no measure here
      p = jnddata.point(:,i);
      f = jnd_measure_trial( p, tol, Nedge );
      jnddata = jnd_sample_update( jnddata, i, f );
      
      % save here to allow for use interuption etc
      save( jndfile, 'jnddata' );  
    end
  end

return;


% Add auxillary data to the jnd model to allow faster use
N = Octree_size( jnddata );
for i = 1:N
  index = Octree_getdata( jnddata, i );
  cornerp = jnddata.point(:, index );
  maxp = max( cornerp, [], 2);
  minp = min( cornerp, [], 2 );
  d1 = cornerp - repmat(minp,1,8);
  d2 = repmat(maxp,1,8) - cornerp;
  logicalp = (d1 >= d2);

  for ii = 1:8
    for jj = 1:8
      if ( jnddata.ordering(:,jj) == logicalp(:,ii) )
        order(ii) = jj;
      end
    end
  end

  if any(order ~= [1:8] )
    break;
  end
  
  jnddata.aux{i}.minp = minp;
  jnddata.aux{i}.midp = (maxp + minp)/2;
  jnddata.aux{i}.maxp = maxp;
  jnddata.aux{i}.dp = maxp - minp;
  
end

save( jndfile, 'jnddata' );  
      
