% RUNPAINT - Painterly Renderer driver for MATLAB
% John Collomosse, University of Bath 2007
% For EPSRC VVG Summer School 2007

close all;
clear all;

%%%%%%%% Add Path %%%%%%%%%%%%
addpath(genpath('./percept'));
addpath(genpath('./painter'));
addpath(genpath('./stb'));
addpath(genpath('./support'));

%%%%%%%% General Parameters %%%%%%%%%

FILE = 'swallowtail.jpg';                      % image to paint
CLASSIFY_FILE='still_life_class.jpg';     % when MODE='salience' load this kpict as the salience map

MODE = 'salience';               % choose one of {'zeros', 'halfhalf', 'ones', 'edgemag', 'classify', 'salience', 'manual'}

EDGEMAG_THRESH=0.5;             % when MODE='edgemag' is a constant of prop. between edge strength and importance
                                % low values e.g. 0.1 give much emphasis to everything, high values e.g. 0.5 give
                                % more balanced emphasis
                                
TARGETSIZE = 640*480;           % resize image to this many pixels  -  don't change for this exercise

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%% Rendering Parameters %%%%%%%%%
                                
PAINTERLY_PARAM.numStrokes=100000;                  % Number of strokes in the painting
PAINTERLY_PARAM.meanShade=1;                        % Colour of each stroke is mean of colour under it (1) or point sampled from seed (0)
PAINTERLY_PARAM.minStrokeThickness=1.0;             % Minimum stroke thickness constant (x2 = no. of pixels wide)
PAINTERLY_PARAM.maxStrokeThickness=15.0;            % Maximum stroke thickness
PAINTERLY_PARAM.minHopExtent=3.0;                   % Minimum hop distance during stroke generation
PAINTERLY_PARAM.maxHopExtent=45.0;                  % Maximum hop distance during stroke generation
PAINTERLY_PARAM.strokeCurvatureLimit=50.0;          % Allow up to this much curvature in a stroke before terminating its formation
PAINTERLY_PARAM.angleVariance=20.0;                 % Introduce noise into hop angle during stroke generation (messy paintings)
PAINTERLY_PARAM.biasStrokes=1;                      % Seed more strokes in more important areas (1) or uniformly distribute strokes (0)
                                
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% 1) Load image, normalise and greyscale
img=imread(FILE);
img=double(img)./255;
resize_ratio=sqrt(TARGETSIZE/(size(img,1)*size(img,2)));
img=imresize(img,round([size(img,1)*resize_ratio,size(img,2)*resize_ratio]),'bicubic');
img(img<0)=0;
img(img>1)=1;
gimg=rgb2gray(img);

% 2) Compute edge information - used for orientation
% and in the case of MODE='edgemag' also for importance map
[edgevect_x edgevect_y]=pictDerivs(gimg,1,1);
directionmap=atan2(edgevect_y,edgevect_x);
edgemag=sqrt(double(edgevect_x).^2 + double(edgevect_y).^2)./sqrt(2);
edgemag(edgemag>EDGEMAG_THRESH)=EDGEMAG_THRESH;


% 3) Generate the importance map 
switch MODE

    case 'zeros'
        importance=zeros(size(gimg));

    case 'halfhalf'
        importance=zeros(size(gimg));
        importance(size(gimg,1)/2:end,:,:)=1;
        
    case 'ones'
        importance=ones(size(gimg));
        
    case 'edgemag'
        importance=pictNorm(edgemag);
        

    case 'classify'        
        importance=double(imread(CLASSIFY_FILE))./255;
        salm=importance(:,:,1);
        salm=imresize(salm,[size(img,1) size(img,2)],'bilinear');
        importance=salm;
        importance(isnan(importance))=0;
        importance=pictNorm(importance).^3;        
        importance=imdilate(importance,strel('disk',7));

    case 'salience'
        stb_img.filename=FILE;
        stb_img.data=uint8(img.*255);
        stb_img.size=size(img);
        stb_img.dims=3;
        stb_img.type='unknown';
        params = defaultSaliencyParams(stb_img.size,'dyadic');
        [salmap,salData] = makeSaliencyMap(stb_img,params);
        salm=salmap.data;
        salm=imresize(salm,[size(img,1) size(img,2)],'bilinear');
        importance=salm;
        importance(isnan(importance))=0;
        importance=pictNorm(importance);        
        importance=imdilate(importance,strel('disk',55));
        
    case 'manual'
        importance = MakeImportanceMap(FILE,resize_ratio);
        
    otherwise
        error('You did not specify a valid MODE for importance map generation');
end 
   
imshow([importance]);
title('Importance map');
drawnow;

%4) Create the painting with the chosen importance map
out=painterly(img,directionmap,importance, PAINTERLY_PARAM);

%5) fixup saturation (paintings can become desaturated due to averaging in
% the painting process) 

% comment this out if you are getting bizarre solarisation effects!

orighsv=rgb2hsv(img);
painthsv=rgb2hsv(out);
avgsat_orig=mean(reshape(orighsv(:,:,2),1,[]));
avgsat_paint=mean(reshape(painthsv(:,:,2),1,[]));
correctedsat=painthsv(:,:,2)+(avgsat_orig-avgsat_paint);
correctedsat(correctedsat>1)=1;
painthsv(:,:,2)=correctedsat;
out=hsv2rgb(painthsv);


%6) Display painting
imshow(out);
title('Result painting');
