function g = Gaussian1D_deriv( x, s, deriv, norm )

% g = Gaussian1D_deriv( x, s, deriv, norm )
%
% Compute a Gaussian derivative at sample points x.
% The basic Gaussian is
%     exp( -0.5*(x/s).^2 )
% The standard deviation, s, is a scalar. All input values must be real.
% The length of the input vector, x, must be at least 2.
% pre:
%    x - regularly spaced sample points, dx = x(i+1)-x(i),  dx > 0, for all i in {1, length(x)-1}
%   s - the standard deviation, a scalar.
%   deriv - (optional) the derivatives to compute indexed as follows:
%     0  zeroth derivative (ie. the Gaussian), the default
%     1  first derivative
%     2  second derivative
%     3  third derivative
%   norm - (optional) the normal to use, can supplied only if deriv is supplied.
%     0  infinity norm
%     1  L_1 norm (default)
%     2  L_2 norm
% post:
%  g - the Gaussian derivative

% set default options, if need to
if nargin < 3
  deriv = 0;
end
if nargin < 4
  norm = 1;
end

dx = x(2)-x(1);       % distance between samples
a = 1/(sqrt(2)*s); % scale factor for a standard Gaussian

% Compute a Gaussian by integrating in a top-hat window around each sample point.
% This tends to be more accurate then straight forward sampling, over all ranges of dx.
y1 = a*(x-dx/2); %  lower-bound of integration region
y2 = a*(x+dx/2); % upper-bound of integration region

switch deriv
case 0, % basic Gaussian
  % Use the fact that erf(x) = 2/sqrt(pi) * integral from 0 to x of  exp(-t^2) dt
  % to compute the integral between samples. This requires a change
  % of variables, as expressed below
  g = sqrt(pi/4)*(erf(y2) - erf(y1)) / (a*dx);
%   switch norm
%   case 1,
%     g = g * (a/sqrt(pi));
%   case 2,
%     g = g / sqrt((1/a)*sqrt(pi/2);
%   end
case 1,
  % the integral of the first deriv is just the Gaussian
  g = (exp( -y2.^2 ) - exp( -y1.^2 ))  / dx;
%   switch norm
%   case 1,
%     g = g / 2 ;
%   case 2,
%     g = g / sqrt(s*sqrt(pi));
%   end
case 2,
  % the integral of the second deriv is just the first deriv
  % recall y = a*x, so a*y = a*a*x, hence ...
  g = (-2*a)*(y2.*exp( -y2.^2 ) - y1.*exp( -y1.^2 )) / dx;
%   switch norm
%   case 1,
%     g = g / (4*sqrt(2)*exp(-0.5)*a);
%   case 2,
%     g = g / sqrt(3*a.^3*sqrt(pi/2));
%   end
case 3,
  a = 2*a*a;
  y1 = y1.^2;
  y2 = y2.^2;
  g = ((2*a*y2 - a).*exp(-y2) - (2*a*y1 - a).*exp(-y1)) / dx;
end

switch norm
case 1,
  g = g / (sum(abs(g))*dx);
case 2,
  g = g / (sqrt(sum(g.*g))*dx);
end

return;
