function [varargout] = pictDerivs( pict,  sigma, order )

% [vararggout] = pictDerivs( pict, sigma, order)
% Compute all partial derivatives of a given order, over all given scales (sigma).
% The pd's are computed using a Gaussian derivative,
% the original (underlying) Gaussian has L_inf norm (so is unity at the origin).
% Applications may scale all derivatives by an appropriate factor to get other norms.
% The number of arguments returned is always order+1.
%
% The output arguments are sorted so that the x derivative order falls
% as the y derivative order rises.; their sum being the total order given.
% For example
%   [u,v] = pictDerivs( p, [0.5, 1, 2, 4, 8], 1 )
% yields the partial derivatives u = dp/dx and v = dp/dy, at scales given.
%   [uu,uv,vv] = pictDerivs( p, [0.5,1,2,4, 8], 2 )
% yields uu = d^2 p/ dx^2, uv = d^2p / dxdy, vv = d^2 p / dy^2.
%
% The input argument order can also be a vector, with each component
% the order of a particular set. For example
%   [u,v,uu,uv,vv] = pictDerivs( p, [0.5, 1, 2, 4, 8], [1 2] );
% is equivalent to the combination of the above examples.
% pre:
%   pict - a gray level image
%   sigma - a vector of scales, each the stabdard deviation of a Gaussian
%   order - a vector of derivative orders
% post
%   varargout - a set of picture derivatives, each named by the application as in the examples above.

[Nr,Nc] = size(pict);
Mr = floor(Nr/2)+1;
Mc = floor(Nc/2)+1;

x = -Mc:(Nc-Mc-1);
y = -Mr:(Nr-Mr-1);

pict = fft2(pict);

% compute the number of output argument at each order
a = zeros(1,length(order));
for j = 1:length(order)
  a(j) = 1+order(j);
end

% compute the number of output arguments in the list BEFORE each order
suma = zeros(1,length(order));
for j = 1:length(order)
  suma(j) = sum(a(1:j-1));
end

% compute the picture derivatives
for i = 1:length(sigma)
  s = sigma(i);
  for j = 1:length(order)
    ord = order(j);
    for k = 0:ord
      g = Gaussian2D_deriv( x,y, s,s, ord-k,k , 0 );
      varargout{ suma(j) + 1+k } (:,:,i)= real( fftshift( ifft2( pict .* fft2(g) ) ) );
    end
  end
end

return;


% Pad the image ready for filtering; this mitigates against boundary artefacts.
% 5 times the half-width of the widest filter is a good choice for the padding size.
[Nr,Nc] = size(pict);
Mr = floor(Nr/2)+1;
Mc = floor(Nc/2)+1;

x = -Mc2:(Nc2-Mc2-1);
y = -Mr2:(Nr2-Mr2-1);


Nsigma = size(sigma,2);
qpictf = fft2(qpict);

for i = 1:Nsigma
  % choose scale - and set a few loop constants
  sigma1 = sigma(i);
  k = 1/(2*pi*sigma1^2);
  
  % 1st partial derivatives
  u = k*Gaussian2D_deriv( x,y, sigma1, sigma1, 1,0, 0 );
  u = real(fftshift(ifft2(  qpictf.*fft2(u)  )));
  u = u(lor:hir, loc:hic);
  v = Gaussian2D_deriv( x,y, sigma1, sigma1, 0,1, 0 );
  v = real(fftshift(ifft2(  qpictf.*fft2( v )  )));
  v = v(lor:hir, loc:hic);
  
  % 2nd partial derivatives
  uu = k*Gaussian2D_deriv( x,y, sigma1, sigma1, 2, 0, 0 );
  uu = real(fftshift(ifft2( qpictf.*fft2(uu) )));
  uu = uu(lor:hir, loc:hic);
  uv = k*Gaussian2D_deriv( x,y, sigma1, sigma1, 1, 1, 0 );
  uv  = real(fftshift(ifft2( qpictf.*fft2( uv ) )));
  uv = uv(lor:hir, loc:hic);
  vv = k*Gaussian2D_deriv( x,y, sigma1, sigma1, 0, 2, 0 );
  vv  = real(fftshift(ifft2( qpictf.*fft2( vv ) )));
  vv = vv(lor:hir, loc:hic);
    
  % 3rd partial derivatives
  uuu = k*Gaussian2D_deriv( x,y, sigma1,sigma1, 3, 0, 0 );
  uuu = real(fftshift(ifft2( qpictf.*fft2( uuu) )));
  uuu = uuu(lor:hir, loc:hic);
  uuv = k*Gaussian2D_deriv( x,y, sigma1,sigma1, 2,1, 0 );
  uuv = real(fftshift(ifft2( qpictf.*fft2( uuv ))));
  uuv = uuv(lor:hir, loc:hic);
  uvv = k*Gaussian2D_deriv( x,y, sigma1,sigma1, 1,2, 0 );
  uvv = real(fftshift(ifft2( qpictf.*fft2( uvv ))));
  uvv = uvv(lor:hir, loc:hic);
  vvv = k*Gaussian2D_deriv( x,y, sigma1,sigma1, 0,3, 0 );
  vvv = real(fftshift(ifft2( qpictf.*fft2( (vvv ))) ));
  vvv = vvv(lor:hir, loc:hic);
  
  % total STEERED derivatives
  a(:,:,i) = atan2( v,u);
  ca = cos(a(:,:,i));
  sa = sin(a(:,:,i));
  ca2 = ca.*ca;
  sa2 = sa.*sa;
  d1(:,:,i) = u.*ca + v.*sa;
  d2(:,:,i) = uu.*ca2 + 2.*uv.*ca.*sa + vv.*sa2;
  d3(:,:,i) = uuu.*ca2.*ca + 3*(uuv.*ca2.*sa + uvv.*ca.*sa2) + vvv.*sa2.*sa;
end

return;
